"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.testSelectorsSnapshotWithFixtures = exports.testReducerSnapshotWithFixtures = exports.testActionSnapshotWithFixtures = exports.testActionSnapshot = exports.testComponentSnapshotsWithFixtures = exports.shallowRenderComponentWithFixtures = void 0;

var _react = _interopRequireDefault(require("react"));

var _enzyme = require("enzyme");

var _enzymeToJson = _interopRequireDefault(require("enzyme-to-json"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

/**
 * Shallow render a component multipile times with fixtures
 * @param  {ReactComponent} Component Component to shallow-render
 * @param  {Object}         fixtures  key=fixture description, value=props to apply
 * @return {Object}                   key=fixture description, value=shallow-rendered component
 */
const shallowRenderComponentWithFixtures = (Component, fixtures) => Object.entries(fixtures).map(([description, props]) => ({
  description,
  component: (0, _enzyme.shallow)(_react.default.createElement(Component, props))
}));
/**
 * Test a component with fixtures and snapshots
 * @param  {ReactComponent} Component Component to test
 * @param  {Object}         fixtures  key=fixture description, value=props to apply
 */


exports.shallowRenderComponentWithFixtures = shallowRenderComponentWithFixtures;

const testComponentSnapshotsWithFixtures = (Component, fixtures) => shallowRenderComponentWithFixtures(Component, fixtures).forEach(({
  description,
  component
}) => it(description, () => expect((0, _enzymeToJson.default)(component)).toMatchSnapshot()));
/**
 * run an action (sync or async) and except the results to much snapshot
 * @param  {Function}  runAction  Action runner function
 * @return {Promise}
 */


exports.testComponentSnapshotsWithFixtures = testComponentSnapshotsWithFixtures;

const testActionSnapshot =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(function* (runAction) {
    const actionResults = runAction(); // if it's an async action

    if (typeof actionResults === 'function') {
      const dispatch = jest.fn();
      yield actionResults(dispatch);
      expect(dispatch.mock.calls).toMatchSnapshot();
    } else {
      expect(actionResults).toMatchSnapshot();
    }
  });

  return function testActionSnapshot(_x) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Test actions with fixtures and snapshots
 * @param  {Object} fixtures key=fixture description, value=action runner function
 */


exports.testActionSnapshot = testActionSnapshot;

const testActionSnapshotWithFixtures = fixtures => Object.entries(fixtures).forEach(([description, runAction]) => it(description, () => testActionSnapshot(runAction)));
/**
 * Test a reducer with fixtures and snapshots
 * @param  {Function} reducer  reducer to test
 * @param  {Object}   fixtures key=fixture description, value=props to apply
 */


exports.testActionSnapshotWithFixtures = testActionSnapshotWithFixtures;

const testReducerSnapshotWithFixtures = (reducer, fixtures) => {
  const reduce = ({
    state,
    action = {}
  } = {}) => reducer(state, action);

  Object.entries(fixtures).forEach(([description, action]) => it(description, () => expect(reduce(action)).toMatchSnapshot()));
};
/**
 * Test selectors with fixtures and snapshots
 * @param  {Object} fixtures  key=fixture description,
 *                            value=selector runner function
 */


exports.testReducerSnapshotWithFixtures = testReducerSnapshotWithFixtures;

const testSelectorsSnapshotWithFixtures = fixtures => Object.entries(fixtures).forEach(([description, selectorRunner]) => it(description, () => expect(selectorRunner()).toMatchSnapshot()));

exports.testSelectorsSnapshotWithFixtures = testSelectorsSnapshotWithFixtures;